/*
 * Copyright (c) 2018 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-BSD-5-Clause-Nordic
 */

#include <zephyr/kernel.h>
//#include <net/socket.h>
#include <stdio.h>
#include <stdlib.h>
//#include <uart.h>
#include <string.h>

#include <zephyr/net/net_ip.h>
#include <zephyr/net/socket.h>
#include <zephyr/net/tls_credentials.h>
#include <zephyr/net/http/client.h>

#define HTTP_HOST "ptsv3.com" 
#define HTTP_PATH "/t/mypts123/post/"
#define HTTP_PORT 80
#define MAX_MTU_SIZE     1000
#define RECV_BUF_SIZE    2048
#define SEND_BUF_SIZE    MAX_MTU_SIZE


#define TEST_STRING	"Hello World!"


#define POST_TEMPLATE "POST %s? HTTP/1.1\r\n"\
                      "Host: %s\r\n"\
                      "Connection: keep-alive\r\n"\
                      "Content-Type: text/plain\r\n"\
                      "Content-length: %d\r\n\r\n"\
                      "%s"


int blocking_recv(int fd, uint8_t *buf, uint32_t size, uint32_t flags)
{
    int err;

	  do {
		    err = recv(fd, buf, size, flags);
	      } while (err < 0 && errno == EAGAIN);

	  return err;
}


int blocking_send(int fd, uint8_t *buf, uint32_t size, uint32_t flags)
{
	int err;

	do {
		err = send(fd, buf, size, flags);
	} while (err < 0 && (errno == EAGAIN));

	return err;
}

int blocking_connect(int fd, struct sockaddr *local_addr, socklen_t len)
{
	int err;

	do {
		err = connect(fd, local_addr, len);
	} while (err < 0 && errno == EAGAIN);

	return err;
}

void app_http_start(void)
{
    struct sockaddr_in local_addr;
    struct addrinfo *res;
    int send_data_len;
    int num_bytes;
    int mtu_size = MAX_MTU_SIZE;
    char send_buf[SEND_BUF_SIZE];
    
    local_addr.sin_family = AF_INET;
    local_addr.sin_port = htons(0);
    local_addr.sin_addr.s_addr = 0;

    printk("\n\nHTTP example\n\r");
    
    int err = getaddrinfo("ptsv3.com", NULL, NULL, &res);

    printk("getaddrinfo err: %d\n\r", err);
    ((struct sockaddr_in *)res->ai_addr)->sin_port = htons(HTTP_PORT);
	
    int client_fd = socket(AF_INET, SOCK_STREAM, 0);

    printk("client_fd: %d\n\r", client_fd);
    err = bind(client_fd, (struct sockaddr *)&local_addr,
		           sizeof(local_addr));
    printk("bind err: %d\n\r", err);
    err = blocking_connect(client_fd, (struct sockaddr *)res->ai_addr,
			       sizeof(struct sockaddr_in));
    printk("connect err: %d\n\r", err);


    printk("\n\rPrepare send buffer:\n\r");

    send_data_len = snprintf(send_buf,
                             mtu_size,
			     			 POST_TEMPLATE, HTTP_PATH,
		             		 HTTP_HOST, strlen(TEST_STRING),
                             TEST_STRING);


    printk("\n\rSend HTTP post request.\n\r");
    do {
	    num_bytes =
		blocking_send(client_fd, send_buf, send_data_len, 0);
		
		if (num_bytes < 0) {
			printk("ret: %d, errno: %s\n", num_bytes, strerror(errno));
		};

    } while (num_bytes < 0);

    printk("\n\rFinished. Closing socket\n");
    err = close(client_fd);
}

int main(void)
{	
    printk("Starting http post...");
    app_http_start();
	
	  return 0;
}